package net.pricefx.common.apibuilder.compensation;

import net.pricefx.common.api.HeaderPhase;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.compensationagreement.CompensationAgreementBuilder;

import java.util.Map;

/**
 *
 *  IMPORTANT: Please read super class notes on sequence of execution!
 *
 */
public class CompensationBuilder extends BasicCompensationBuilder<CompensationBuilder> implements CompensationAgreementBuilder {

    // read only state, don't get propagated further, it is here for user convenience
    private final Object compensationView;
    private final CompensationHelper helper;

    public CompensationBuilder(String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations, Object compensationView, HeaderPhase phase) {
        super(instanceId, classId, idGenerator, invocations, phase);
        this.compensationView = compensationView;
        this.helper = new CompensationHelper((Map<String, Object>)compensationView);
    }

    /**
     * Gets the full compensation object as a nested map. This map contains all header inputs/outputs, and all line items (with their inputs/outputs).
     *
     * @return The compensation object
     */
    public Object getCompensationView() {
        addInvocation("getCompensationView");
        return compensationView;
    }

    /**
     * Gets a helper class that assists in manipulating the compensation object. You can use it instead of directly reading the CompensationView map.
     *
     * @return The helper object
     */
    public CompensationHelper getHelper() {
        addInvocation("getHelper");
        return helper;
    }

    /**
     * Applies a compensation structure. That usually means adding a set of folders and/or items.<p>
     * Example:
     * <blockquote><pre>
     *      if (coProcessor.isPrePhase()) {
     *       def structure = new CompensationStructure()
     *       productSubCategory.each {
     *            structure.addFolder(it.attribute2)
     *       }
     *       coProcessor.addCompensationStructure(structure)
     *  }
     * </pre></blockquote>
     *
     *
     * @param structure
     * @return
     */
    public CompensationBuilder addCompensationStructure(CompensationStructure structure) {
        addInvocation("addCompensationStructure", structure);
        return this;
    }

    /**
     * Creates and adds a new folder under the ROOT folder.
     * <p>
     * If you intend to add many folders and items, use the CompensationStructure approach instead.
     *
     */
    public void addFolder(String label) {
        CompensationStructure s = new CompensationStructure();
        s.addFolder(label);
        addCompensationStructure(s);
    }
}
