package net.pricefx.common.apibuilder.customform;

import net.pricefx.common.api.HeaderPhase;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.clicoll.CalculableLineItemCollectionBuilder;
import net.pricefx.common.apibuilder.clicoll.LineItemHelper;

import java.util.Map;

/**
 * CustomFormCollectionBuilder is the starting point of an interface that allows you manipulate
 * a custom form object in a custom form header calculation logic.<br />
 * The header calculation is executed twice: Once as 'pre phase' and once after that as 'post phase'.<br />
 * Use <code>isPrePhase()</code> and <code>isPostPhase()</code> methods to detect current phase. <br />
 * The custom form builder is available in the header logic by this call:<br />
 * <code>def processor = customFormProcessor</code>
 * <br /><br />
 * IMPORTANT: Please read super class notes on sequence of execution!
 *
 */
public class CustomFormCollectionBuilder extends CalculableLineItemCollectionBuilder<CustomFormCollectionBuilder> {
    private final LineItemHelper lineItemHelper;

    public CustomFormCollectionBuilder(String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations, Map<String, Object> view, HeaderPhase phase) {
        super(instanceId, classId, idGenerator, invocations, phase);
        this.lineItemHelper = new LineItemHelper(view);
    }

    /**
     * Creates an instance of {@link CustomFormBuilderResult} with the invocations accumulated in this builder instance
     * @return a {@link CustomFormBuilderResult} instance
     */
    @Override
    public CustomFormBuilderResult build() {
        addInvocation(METHOD_BUILD);

        CustomFormBuilderResult result = new CustomFormBuilderResult();
        result.invocations = invocations.get();

        return result;
    }

    /**
     * Gets a helper class that assists in manipulating the custom form
     * @return The helper object
     */
    public LineItemHelper getHelper() {
        addInvocation("getHelper");
        return lineItemHelper;
    }
}
