package net.pricefx.server.dto.calculation;

import com.fasterxml.jackson.annotation.JsonProperty;
import net.pricefx.common.api.CalculationResultType;
import net.pricefx.domain.CompensationAgreementRecord;
import net.pricefx.domain.CompensationRecord;
import net.pricefx.domain.CompensationRecordSet;

public class ResultCompensationRecord implements ComplexCalculationResult, ResultCompensationAgreementRecord  {

    private static final long serialVersionUID = -2863126236071806981L;

    @JsonProperty public String uniqueName;
    @JsonProperty public String name;
    @JsonProperty public String compensationRecordSetLabel;
    @JsonProperty public String description;
    @JsonProperty public String sourceId;		// the ID of the compensation this record belongs to

    protected ResultCompensationRecord() {}

    public ResultCompensationRecord(final CompensationAgreementRecord compensationAgreementRecord) {
        uniqueName = compensationAgreementRecord.getUniqueName();
        name = compensationAgreementRecord.getName();
        compensationRecordSetLabel = compensationAgreementRecord.getRecordSetLabel();
        sourceId = compensationAgreementRecord.getSourceId();
    }

    @Override
    public String getResultType() {
        return CalculationResultType.COMPENSATIONRECORD.name();
    }

    public String getDescription() {
        if (description == null){
            boolean hasName = name != null && !"".equals(name);
            boolean hasSet = compensationRecordSetLabel != null && !"Default".equals(compensationRecordSetLabel);
            if (hasName && hasSet){
                description = String.format("%s: %s (%s)", uniqueName, name, compensationRecordSetLabel);
            }
            else if (hasName){
                description = String.format("%s: %s", uniqueName, name);
            }
            else if (hasSet){
                description = String.format("%s: %s", uniqueName, compensationRecordSetLabel);
            }
            else description = new String(uniqueName);
        }
        return description;
    }

    @Override
    public String toString() {
        return getDescription();
    }
}
