package net.pricefx.server.protocol;

import java.io.Closeable;
import java.io.Serializable;

/**
 * Unifying (and very limited) JDBC ResultSet like API to a forward-only cursor on a query result, either from a Hibernate or PA query.
 */
public interface StreamResults extends Serializable, Closeable {
	
    /**
     * Moves the cursor forward one row from its current position.
     * A cursor is initially positioned before the first row; the first call to the method <code>next</code> makes the first row the current row; 
     * the second call makes the second row the current row, and so on.
     *
     * @return <code>true</code> if the new current row is valid; <code>false</code> if there are no more rows.
     */
	boolean next() throws Exception;

	/**
	 * Retrieves the current row, which depending on the query executed could be anything from a <code>PersistedObject</code> to 
	 * a map of (field,value) pairs to a single primitive value. 
	 */
	Object get() throws Exception;
	
	/**
	 * Releases the often significant resources required to provide access to the query result set.
	 * If not closed this could lead to memory leaks, and ultimately system instability or even a system crash.  
	 */
	void close();
	
}
